import unittest
import json
import os

from emodpy_generic.interventions import polio_vaccine
from emodpy_generic.interventions import SimpleSIA
from emodpy_generic.interventions import emergence
from emodpy_generic.interventions import complex_import
import emod_api.campaign as camp


class TestGenericInterventions(unittest.TestCase):

    # region helper methods
    def setUp(self) -> None:
        self.tmp_intervention=None
        self.is_debugging = True
        self.iv_name=None
        self.timestep=None
        self.coverage=None
        self.genome=None
        self.clade = None
        current_directory = os.path.dirname(os.path.realpath(__file__))
        camp.schema_path = os.path.join(current_directory, "./inputs/bin/schema.json")
        self.camp = camp

        return

    def tearDown(self) -> None:
        if self.is_debugging:
            self.write_debug_files()
        return

    def write_debug_files(self):
        with open(f'DEBUG_Simple_IPV.json', 'w') as outfile:
            json.dump(self.tmp_intervention, outfile, indent=4, sort_keys=True)
        return

    def parse_intervention(self):
        self.event_coordinator = self.tmp_intervention["Event_Coordinator_Config"]
        self.intervention_config = self.event_coordinator["Intervention_Config"]

    def check_changed_values_IPV(self):
        self.assertEqual(self.event_coordinator["Intervention_Config"]["Actual_IndividualIntervention_Config"]["Actual_IndividualIntervention_Configs"][0]['Intervention_Name'], self.iv_name)
        self.assertEqual(self.event_coordinator["Intervention_Config"]["Demographic_Coverage"], self.coverage)
        self.assertEqual(self.tmp_intervention["Start_Day"], self.timestep+1)

    def test_simple_ipv1(self):
        self.iv_name = "IPV1"
        self.timestep=4
        self.coverage=0.9
        self.assertIsNone(self.tmp_intervention)

        self.tmp_intervention = polio_vaccine.SimpleIPV1(camp=self.camp, timestep=self.timestep,coverage=self.coverage)
        
        self.parse_intervention()
        self.check_changed_values_IPV()

    def test_simple_ipv2(self):
        self.iv_name = "IPV2"
        self.timestep=10
        self.coverage=0.5

        self.assertIsNone(self.tmp_intervention)

        self.tmp_intervention = polio_vaccine.SimpleIPV2(camp=self.camp, timestep=self.timestep, coverage=self.coverage)
        
        self.parse_intervention()
        self.check_changed_values_IPV()

    def check_changed_values_SIA(self):
        self.assertEqual(self.intervention_config['Intervention_Name'], self.iv_name)
        self.assertEqual(self.event_coordinator["Demographic_Coverage"], self.coverage)
        self.assertEqual(self.tmp_intervention["Start_Day"], self.timestep+1)
        self.assertEqual(self.event_coordinator['Target_Age_Max'], self.target_age_max)
        self.assertEqual(self.event_coordinator['Target_Age_Min'], self.target_age_min)
        self.assertEqual(self.tmp_intervention['Event_Name'], self.event_name)
        self.assertEqual(self.tmp_intervention['Nodeset_Config']['class'], self.nodeset)


    def test_sia_default(self):
        self.tmp_intervention = SimpleSIA(camp=self.camp)
        self.coverage=1
        self.timestep=1
        self.iv_name = 'Vaccine'
        self.target_age_max = 5.0
        self.target_age_min = 0.75
        self.event_name = 'SIA'
        # Should be nodesetall since it is empty
        self.nodeset = 'NodeSetAll'
        self.parse_intervention()
        self.check_changed_values_SIA()

    def test_sia_custom(self):
        self.coverage=0.8
        self.timestep=4
        self.iv_name = 'Vaccine'
        self.target_age_max = 4.0
        self.target_age_min = 1.0
        self.event_name = 'BoomPow'
        # Should be nodesetall since it is empty
        self.nodeset = 'NodeSetAll'
        self.tmp_intervention = SimpleSIA(camp=self.camp, Event_Name=self.event_name, 
                                          timestep=self.timestep,
                                          Target_Age_Min=self.target_age_min,
                                          Target_Age_Max=self.target_age_max,
                                          Coverage=self.coverage)
        self.parse_intervention()
        self.check_changed_values_SIA()

    def check_changed_values_Emergence(self):
        self.assertEqual(self.tmp_intervention['Nodeset_Config']['class'], self.nodeset)
        self.assertEqual(self.tmp_intervention["Start_Day"], self.timestep)
        self.assertEqual(self.intervention_config["Clade"], self.clade)
        self.assertEqual(self.intervention_config["Genome"], self.genome)


    def test_emergence_default(self):
        self.genome=0
        self.clade=0
        self.timestep=1
        self.nodeset='NodeSetAll'
        self.tmp_intervention=emergence.basicEmergenceEvent(self.camp, self.genome,self.clade,self.timestep,[])
        self.parse_intervention()
        self.check_changed_values_Emergence()

    def test_emergence_custom(self):
        self.genome=1
        self.clade=1
        self.timestep=4
        self.nodeset='NodeSetAll'
        self.tmp_intervention=emergence.basicEmergenceEvent(self.camp, self.genome,self.clade,self.timestep,[])
        self.parse_intervention()
        self.check_changed_values_Emergence()

    def check_changed_values_basic_SIA(self):
        self.assertEqual(self.tmp_intervention['Nodeset_Config']['class'], self.nodeset)
        self.assertEqual(self.tmp_intervention["Start_Day"], self.timestep)
        self.assertEqual(self.intervention_config["Clade"], self.clade)
        self.assertEqual(self.intervention_config["Genome"], self.genome)
        self.assertEqual(self.event_coordinator["Demographic_Coverage"], self.coverage)

    def test_basic_default_SIA(self):
        self.genome=0
        self.clade=0
        self.timestep=1
        self.nodeset='NodeSetAll'
        self.coverage = 1.0
        self.tmp_intervention=emergence.basicSIAEvent(self.camp, self.coverage, self.genome,self.clade,self.timestep,[])
        self.parse_intervention()
        self.check_changed_values_basic_SIA()

    def test_basic_custom_SIA(self):
        self.genome=1
        self.clade=1
        self.timestep=3
        self.nodeset='NodeSetAll'
        self.coverage = 0.6
        self.tmp_intervention=emergence.basicSIAEvent(self.camp, self.coverage, self.genome,self.clade,self.timestep,[])
        self.parse_intervention()
        self.check_changed_values_basic_SIA()

    def check_changed_values_complex_import(self):
        self.assertEqual(self.intervention_config['Daily_Import_Pressures'], self.dips)
        self.assertEqual(self.intervention_config['Durations'], self.durs)
        self.assertEqual(self.tmp_intervention["Start_Day"], self.timestep+1)
        self.assertEqual(self.tmp_intervention['Nodeset_Config']['class'], self.nodeset)

    def test_default_complex_import(self):
        self.dips = [1/(774*10)]
        self.durs = [100000]
        self.timestep = 1
        self.nodeset = 'NodeSetAll'

        self.tmp_intervention = complex_import.ComplexImportationEvent(camp=self.camp)
        self.parse_intervention()
        self.check_changed_values_complex_import()

    def test_custom_complex_import(self):
        self.dips = [1/(774*12)]
        self.durs = [100007]
        self.timestep = 2
        self.nodeset = 'NodeSetAll'

        self.tmp_intervention = complex_import.ComplexImportationEvent(camp=self.camp, dips=self.dips, durs=self.durs, timestep=self.timestep)
        self.parse_intervention()
        self.check_changed_values_complex_import()


if __name__ == "__main__":
    unittest.main()
